/* See LICENSE file for copyright and license details. */

//The media and volume keys
//usr/include/X11/XF86keysym.h
#include "X11/XF86keysym.h"

/* appearance */
static const unsigned int borderpx  = 0;        /* border pixel of windows */
static const unsigned int gappx     = 3;        /* gaps between windows */
static const unsigned int snap      = 32;       /* snap pixel */
static const int showbar            = 0;        /* 0 means no bar */
static const int topbar             = 1;        /* 0 means bottom bar */
static const char *fonts[]          = { "Iosevka:size=10" };
static const char dmenufont[]       = "Iosevka:size=10";
static const char col_gray1[]       = "#222222";
static const char col_gray2[]       = "#444444";
static const char col_gray3[]       = "#bbbbbb";
static const char col_gray4[]       = "#eeeeee";
static const char col_cyan[]        = "#005577";
static const char *colors[][3]      = {
	/*               fg         bg         border   */
	[SchemeNorm] = { col_gray3, col_gray1, col_gray2 },
	[SchemeSel]  = { col_gray4, col_cyan,  col_cyan  },
};

/* tagging */
static const char *tags[] = { "1", "2", "3", "4", "5", "6", "7", "8", "9" };

static const Rule rules[] = {
	/* xprop(1):
	 *	WM_CLASS(STRING) = instance, class
	 *	WM_NAME(STRING) = title
	 */
	/* class                  instance    title                 tags mask     isfloating   isterminal noswallow  nofakefullscreen  iskeyboard  monitor */
	//{ "Gimp",               NULL,       NULL,                 0,            0,           0,         0,         0,                0,          -1 },
	//{ "Firefox",            NULL,       NULL,                 1 << 8,       0,           0,         0,         0,                0,          -1 },
	{ "xterm-256color",       NULL,       NULL,                 0,            0,           1,         1,         0,                0,          -1 },
	{ "Thunderbird",          NULL,       NULL,                 1 << 8 ,      0,           0,         0,         0,                0,          -1 },
	{ NULL,                   NULL,       "noswallow",          0,            0,           0,         1,         0,                0,          -1 },
    { "svkbd",                NULL,       NULL,                 0,            1,           0,         1,         0,                1,          -1 },
    { "Onboard-settings",     NULL,       NULL,                 0,            0,           0,         0,         0,                0,          -1 }

};

/* layout(s) */
static const float mfact     = 0.55; /* factor of master area size [0.05..0.95] */
static const int nmaster     = 1;    /* number of clients in master area */
static const int resizehints = 0;    /* 1 means respect size hints in tiled resizals */
static int attachbelow = 0;    /* 1 means attach after the currently active window */

#include "tcl.c"
#include "fibonacci.c"
static const Layout layouts[] = {
	/* symbol     arrange function */
	{ "[]=",      tile },    /* first entry is default */
	{ "><>",      NULL },    /* no layout function means floating behavior */
	{ "[M]",      monocle },
	{ "|||",      tcl },
 	{ "[@]",      spiral },
 	{ "[\\]",     dwindle },
};

/* key definitions */
#define MODKEY Mod4Mask
#define TAGKEYS(KEY,TAG) \
	{ MODKEY,                       KEY,   0,      view,           {.ui = 1 << TAG} }, \
	{ MODKEY|ControlMask,           KEY,   0,      toggleview,     {.ui = 1 << TAG} }, \
	{ MODKEY|ShiftMask,             KEY,   0,      tag,            {.ui = 1 << TAG} }, \
	{ MODKEY|ControlMask|ShiftMask, KEY,   0,      toggletag,      {.ui = 1 << TAG} },

/* helper for spawning shell commands in the pre dwm-5.0 fashion */
#define SHCMD(cmd) { .v = (const char*[]){ "/bin/sh", "-c", cmd, NULL } }

/* commands */
static char dmenumon[2] = "0"; /* component of dmenucmd, manipulated in spawn() */
static const char *dmenucmd[] = { "launcher", NULL };
//static const char *dmenucmd[] = { "dmenu_run", "-m", dmenumon, "-fn", dmenufont, "-nb", col_gray1, "-nf", col_gray3, "-sb", col_cyan, "-sf", col_gray4, NULL };
static const char *termcmd[]  = { "folder-shell", NULL };

static const char *fullscreenshot[] = { "screenshot",  NULL };
static const char *activescreenshot[] = { "screenshot", "window", NULL };
static const char *selectscreenshot[] = { "screenshot", "select", NULL };

static const char *selectscreenrecord[] = { "screenrecord", "select", NULL };

static const char *chromium[] = { "chromium", NULL };
static const char *firefox[] = { "firefox-developer-edition", NULL };

static const char *date[] = { "datetime", NULL };
static const char *battery[] = { "battery", NULL };

static const char *unicode[] = { "unicode-character-select", NULL };
static const char *youtube[] = { "open-youtube", NULL };

static const char *playpause[] = { "playerctl", "play-pause", NULL };

static const char *logout[] = { "rofi-shutdown", NULL };

static const char *manPages[] = { "man-page-pdf", NULL };
static const char *ports[] = { "ports", NULL };

static const char *project[] = { "project", "switch", "--rofi", NULL };
static const char *whichproject[] = { "project", "current", "--notify", NULL };

static const char *screenlayout[] = { "screenlayout", NULL };

static const char *pass[] = { "password-manager", NULL };

static const char *volumeUp[] = { "volume", "up", NULL };
static const char *volumeDown[] = { "volume", "down", NULL };
static const char *volumeToggle[] = { "volume", "toggle", NULL };

static const char *brightnessUp[] = { "brightness", "up", NULL };
static const char *brightnessDown[] = { "brightness", "down", NULL };


static const char *lockScreen[] = { "screenlock", "--suspend", NULL };
static const char *powerMenu[] = { "rofi-shutdown", NULL };

static const char *toggleKeyboard[] = { "toggleKeyboard", NULL };

static const char *actOnLast[] = { "actOnLast", NULL };
static const char *actOnLastDefault[] = { "actOnLast", "--first", NULL };

static const char *dunstClose[] = { "dunstctl", "close", NULL };
static const char *dunstOpenLast[] = { "dunstctl", "history-pop", NULL };

#include "movestack.c"

#define MULTIKEY_THRESHOLD_MS_PRESS 200
#define MULTIKEY_THRESHOLD_MS_HOLD 700


static Key keys[] = {
	/* modifier                     key                        count      function        argument */
	{ MODKEY,                       XK_Return,                 0,         spawn,          {.v = termcmd } },
	//{ MODKEY|ShiftMask,             XK_Return,                 0,         zoom,           {0} },

	{ MODKEY,                       XK_Tab,                    0,         toggleAttachBelow,           {0} },
	{ MODKEY|ShiftMask,             XK_Tab,                    0,         spawn,           {.v = dunstOpenLast} },

	{ MODKEY,                       XK_space,                  0,         spawn,           {.v = dunstClose} },

	{ MODKEY|ShiftMask,             XK_a,                      0,         spawn,          {.v = screenlayout } },

	{ MODKEY,                       XK_b,                      0,         togglebar,      {0} },
	{ MODKEY|ShiftMask,             XK_b,                      0,         spawn,          {.v = battery} },

	{ MODKEY,                       XK_c,                      0,         spawn,          {.v = firefox } },
	{ MODKEY|ShiftMask,             XK_c,                      0,         spawn,          {.v = chromium } },

	{ MODKEY,                       XK_d,                      0,         spawn,          {.v = pass } },
	{ MODKEY|ShiftMask,             XK_d,                      0,         spawn,          {.v = date } },

	{ MODKEY,                       XK_f,                      0,         setlayout,      {.v = &layouts[4]} },
	{ MODKEY|ShiftMask,             XK_f,                      0,         setlayout,      {.v = &layouts[5]} },

	{ MODKEY,                       XK_h,                      0,         setmfact,       {.f = -0.05} },

	{ MODKEY,                       XK_i,                      0,         incnmaster,     {.i = +1 } },
	{ MODKEY|ShiftMask,             XK_i,                      0,         incnmaster,     {.i = -1 } },

	{ MODKEY,                       XK_j,                      0,         focusstack,     {.i = +1 } },
	{ MODKEY|ShiftMask,             XK_j,                      0,         movestack,      {.i = +1 } },

	{ MODKEY,                       XK_k,                      0,         focusstack,     {.i = -1 } },
	{ MODKEY|ShiftMask,             XK_k,                      0,         movestack,      {.i = -1 } },

	{ MODKEY,                       XK_l,                      0,         setmfact,       {.f = +0.05} },
	{ MODKEY|ShiftMask,             XK_l,                      0,         spawn,          {.v = logout} },

	{ MODKEY,                       XK_m,                      0,         setlayout,      {.v = &layouts[2]} },
	{ MODKEY|ShiftMask,             XK_m,                      0,         spawn,          {.v = manPages } },

	{ MODKEY,                       XK_o,                      0,         spawn,          {.v = actOnLast } },
	{ MODKEY|ShiftMask,             XK_o,                      0,         spawn,          {.v = actOnLastDefault } },

	{ MODKEY,                       XK_p,                      0,         spawn,          {.v = dmenucmd } },
	{ MODKEY|ShiftMask,             XK_p,                      0,         spawn,          {.v = ports } },

	{ MODKEY,                       XK_q,                      0,         killclient,     {0} },

	{ MODKEY|ShiftMask,             XK_r,                      0,         quit,           {0} },

	{ MODKEY,                       XK_t,                      0,         setlayout,      {.v = &layouts[0]} },
	{ MODKEY|ShiftMask,             XK_t,                      0,         setlayout,      {.v = &layouts[3]} },

	{ MODKEY,                       XK_u,                      0,         spawn,          {.v = unicode } },

	{ MODKEY,                       XK_w,                      0,         spawn,          {.v = whichproject } },
	{ MODKEY|ShiftMask,             XK_w,                      0,         spawn,          {.v = project } },

	{ MODKEY,                       XK_y,                      0,         spawn,          {.v = youtube } },

	{ MODKEY,                       XK_comma,                  0,         focusmon,       {.i = -1 } },
	{ MODKEY|ShiftMask,             XK_comma,                  0,         tagmon,         {.i = -1 } },

	{ MODKEY,                       XK_period,                 0,         focusmon,       {.i = +1 } },
	{ MODKEY|ShiftMask,             XK_period,                 0,         tagmon,         {.i = +1 } },

	{ 0,                            XK_Print,                  0,         spawn,          {.v = fullscreenshot } },
	{ ControlMask,                  XK_Print,                  0,         spawn,          {.v = activescreenshot } },
	{ ShiftMask,                    XK_Print,                  0,         spawn,          {.v = selectscreenshot } },

	{ 0,                            XK_F10,                    0,         spawn,          {.v = fullscreenshot } },
	{ ControlMask,                  XK_F10,                    0,         spawn,          {.v = activescreenshot } },
	{ ShiftMask,                    XK_F10,                    0,         spawn,          {.v = selectscreenshot } },

	{ 0,                            XF86XK_AudioPlay,          0,         spawn,          {.v = playpause } },
	{ 0,                            XF86XK_AudioPause,         0,         spawn,          {.v = playpause } },
	{ 0,                            XF86XK_AudioRaiseVolume,   0,         spawn,          {.v = volumeUp } },
	{ 0,                            XF86XK_AudioLowerVolume,   0,         spawn,          {.v = volumeDown } },
	{ 0,                            XF86XK_AudioMute,          0,         spawn,          {.v = volumeToggle } },
	{ 0,                            XF86XK_MonBrightnessUp,    0,         spawn,          {.v = brightnessUp } },
	{ 0,                            XF86XK_MonBrightnessDown,  0,         spawn,          {.v = brightnessDown } },

	{ MODKEY,                       XK_Right,                  0,         shiftview,      {.i = 1 } },
	{ MODKEY,                       XK_Left,                  0,         shiftview,      {.i = -1 } },
	// On the pinephone, I want a single press of the power button to put the phone in sleep mode.
	// I want a double press to toggle the keyboard
	// I want a hold press to bring up the power menu
	// TODO: Make a command for screen lock
	{ 0,                            XF86XK_PowerOff,           1,         spawn,          {.v = lockScreen } },
	{ 0,                            XF86XK_PowerOff,           2,         spawn,          {.v = toggleKeyboard } },
	{ 0,                            XF86XK_PowerOff,           3,         spawn,          {.v = powerMenu } },

	TAGKEYS(                        XK_1,                                      0)
	TAGKEYS(                        XK_2,                                      1)
	TAGKEYS(                        XK_3,                                      2)
	TAGKEYS(                        XK_4,                                      3)
	TAGKEYS(                        XK_5,                                      4)
	TAGKEYS(                        XK_6,                                      5)
	TAGKEYS(                        XK_7,                                      6)
	TAGKEYS(                        XK_8,                                      7)
	TAGKEYS(                        XK_9,                                      8)

	{ MODKEY,                       XK_0,                      0,         view,           {.ui = ~0 } },
	{ MODKEY|ShiftMask,             XK_0,                      0,         tag,            {.ui = ~0 } },
};

/* button definitions */
/* click can be ClkTagBar, ClkLtSymbol, ClkStatusText, ClkWinTitle, ClkClientWin, or ClkRootWin */
static Button buttons[] = {
	/* click                event mask      button          function        argument */
	{ ClkLtSymbol,          0,              Button1,        setlayout,      {0} },
	{ ClkLtSymbol,          0,              Button3,        setlayout,      {.v = &layouts[2]} },
	{ ClkWinTitle,          0,              Button2,        zoom,           {0} },
	{ ClkStatusText,        0,              Button2,        spawn,          {.v = termcmd } },
	{ ClkClientWin,         MODKEY,         Button1,        movemouse,      {0} },
	{ ClkClientWin,         MODKEY,         Button2,        togglefloating, {0} },
	{ ClkClientWin,         MODKEY,         Button3,        resizemouse,    {0} },
	{ ClkTagBar,            0,              Button1,        view,           {0} },
	{ ClkTagBar,            0,              Button3,        toggleview,     {0} },
	{ ClkTagBar,            MODKEY,         Button1,        tag,            {0} },
	{ ClkTagBar,            MODKEY,         Button3,        toggletag,      {0} },
};

